import os
from config.settings import Config
from dataclasses import dataclass
from supabase import create_client, Client


URL = Config.SUPABASE_URL
KEY = Config.SUPABASE_KEY

supabase: Client = create_client(URL, KEY)


@dataclass
class PowerReading:
    node_id: int
    instantaneous_power: float
    min_power: float
    max_power: float
    avg_power: float
    sample_period: str
    power_state: str
    period: str
    measurement_time: str


def create_power_reading_global(reading: PowerReading) -> None:
    """
    Inserts a new power reading into the Supabase database.

    Args:
        node_id (str): Unique identifier for the device.
        instantaneous_power (float, optional): Instantaneous power measurement. Defaults to 0.
        min_power (float, optional): Minimum power measurement. Defaults to 0.
        max_power (float, optional): Maximum power measurement. Defaults to 0.
        avg_power (float, optional): Average power measurement. Defaults to 0.
        sample_period (str, optional): Time period of the measurement. Defaults to "".
        power_state (str, optional): Power state (e.g., "on", "off"). Defaults to "".

    Returns:
        None
    """

    if not reading.node_id:
        raise ValueError("node_id is required")

    try:
        # Create a data dictionary with the provided arguments
        data = {
            "node_id": reading.node_id,
            "instantaneous_power": reading.instantaneous_power,
            "min_power": reading.min_power,
            "max_power": reading.max_power,
            "avg_power": reading.avg_power,
            "sample_period": reading.sample_period,
            "power_state": reading.power_state,
            "period": reading.period,
            "measurement_time": reading.measurement_time,
        }

        # Insert the data into the Supabase database
        response = supabase.table("power_readings").insert(data).execute()
        # print(response.data[0]["id"])
    except Exception as e:
        print("Error saving the data in the centralized database: ", e)



import argparse
from .scripts import simplev7, get_power_info
from .modules.utilities import create_tables_if_not_exists
from . import install_cron


def run_default():
    create_tables_if_not_exists()
    simplev7.main()
    get_power_info.main()


def main():
    parser = argparse.ArgumentParser(description="power_corn CLI")
    subparsers = parser.add_subparsers(dest="command")

    subparsers.add_parser("install-cron", help="Install cron tasks for power_corn")

    args = parser.parse_args()

    if args.command == "install-cron":
        install_cron.main()
    else:
        run_default()



import subprocess
import os
import shutil


def main():
    log_dir = os.path.expanduser("~/.local/share/power_corn/logs")
    os.makedirs(log_dir, exist_ok=True)

    # Detect absolute path of power_corn
    power_corn_path = shutil.which("power_corn")
    if not power_corn_path:
        print("Error: 'power_corn' not in PATH.")
        return

    CRON_LINES = f"""
* * * * * {power_corn_path} >> {log_dir}/power.log 2>&1
* * * * * sleep 20; {power_corn_path} >> {log_dir}/power.log 2>&1
* * * * * sleep 40; {power_corn_path} >> {log_dir}/power.log 2>&1
    """

    try:
        current_cron = subprocess.check_output(
            ["crontab", "-l"], stderr=subprocess.DEVNULL
        ).decode()
    except subprocess.CalledProcessError:
        current_cron = ""

    if CRON_LINES.strip() in current_cron:
        print("Cronjobs already installed.")
        return

    new_cron = current_cron.strip() + "\n" + CRON_LINES.strip() + "\n"
    proc = subprocess.run(["crontab", "-"], input=new_cron.encode())

    if proc.returncode == 0:
        print("Cronjobs installed successfully.")
    else:
        print("Error installing cronjobs.")


if __name__ == "__main__":
    main()



import os


class Config:
    DB_PATH = os.path.join(
        os.path.expanduser("~"),
        ".local",
        "share",
        "power_corn",
        "power_timeseries.sqlite",
    )
    SUPABASE_URL = os.getenv("SUPABASE_URL")
    SUPABASE_KEY = os.getenv("SUPABASE_KEY")
    NODE_ID = os.getenv("NODE_ID")



from src.modules.utilities import get_power_readings_now, get_power_readings_hour


def main():
    last_power = get_power_readings_now()
    avg_power_hour = get_power_readings_hour()
    # print(f"{last_power},{avg_power_hour}")



import unittest
from src.modules.utilities import parse_and_save_ipmi_output


class TestUtilities(unittest.TestCase):
    def test_parse_and_save_ipmi_output(self):
        sample_output = """
        Instantaneous power reading: 150 Watts
        Minimum during sampling period: 100 Watts
        Maximum during sampling period: 200 Watts
        Average power reading over sample period: 125 Watts
        Sampling period: 10 Seconds.
        Power reading state is: active
        """
        result = parse_and_save_ipmi_output(sample_output, "now", "2025-01-01")
        self.assertEqual(result["Instantaneous power reading"], "150 Watts")


if __name__ == "__main__":
    unittest.main()



import os
import re
import sqlite3
from config.settings import Config
from src.modules.supabasefuncs import create_power_reading_global, PowerReading

DB_PATH = Config.DB_PATH


def parse_and_log_ipmi_output(output, sample_period_label, timestamp):
    """Parse the output of ipmitool and save it in a log file."""
    if not output:
        print("Warning: Void output of ipmitool.")
        return {}

    log_dir = os.path.join(
        os.path.expanduser("~"), ".local", "share", "power_corn", "logs"
    )
    os.makedirs(log_dir, exist_ok=True)

    log_path = os.path.join(log_dir, f"{sample_period_label}_{timestamp}.txt")

    try:
        with open(log_path, "w") as file:
            file.write(output)
    except IOError as e:
        print(f"Error saving the log {log_path}: {e}")

    parsed_data = {}
    lines = output.strip().splitlines()

    for line in lines:
        # Busca todos los pares clave: valor, incluso si hay más de uno por línea
        matches = re.findall(
            r"([A-Za-z\s]+?):\s+([^\n]+?)(?=(?:\s{2,}[A-Za-z\s]+?:|$))", line
        )
        for key, value in matches:
            clean_key = key.strip()
            clean_value = value.strip().rstrip(".")  # elimina el punto final si hay
            parsed_data[clean_key] = clean_value

    return parsed_data


def save_to_db(parsed_data, period, timestamp):
    """
    Save the parsed data to the database.

    This function takes the parsed data from the ipmitool output and saves it
    to the power_readings table in the SQLite database. It also creates a record
    in the global database using the `create_power_reading_global` function.

    Args:
        parsed_data (dict): The parsed data from the ipmitool output.
        period (str): The sample period, either "now" or "1_hour".
        timestamp (float): The timestamp of the power reading.

    Returns:
        None
    """

    with sqlite3.connect(DB_PATH) as conn:
        cursor = conn.cursor()

        NODE_ID = Config.NODE_ID
        data = extract_power_data(parsed_data, NODE_ID, period, timestamp)

        cursor.execute(
            """
            INSERT INTO power_readings
            (timestamp, instantaneous_power, min_power, max_power, avg_power, sample_period, power_state, period)
            VALUES (?, ?, ?, ?, ?, ?, ?, ?)
            """,
            (
                data.measurement_time,
                data.instantaneous_power,
                data.min_power,
                data.max_power,
                data.avg_power,
                data.sample_period,
                data.power_state,
                period,  # "now" o "1_hour"
            ),
        )

        # create the records in the global database
        create_power_reading_global(data)


def get_latest_avg_power(period):
    """Gets the latest value of avg_power for a given period ('now' or '1_hour')."""
    conn = sqlite3.connect(DB_PATH)
    cursor = conn.cursor()
    cursor.execute(
        "SELECT avg_power FROM power_readings WHERE period = ? ORDER BY timestamp DESC LIMIT 1",
        (period,),
    )
    result = cursor.fetchone()
    conn.close()
    return result[0] if result else None


def get_power_readings_now():
    return get_latest_avg_power("now")


def get_power_readings_hour():
    return get_latest_avg_power("1_hour")


def create_tables_if_not_exists():

    os.makedirs(os.path.dirname(DB_PATH), exist_ok=True)

    try:

        conn = sqlite3.connect(DB_PATH)
        cursor = conn.cursor()

        cursor.execute(
            """
        CREATE TABLE IF NOT EXISTS power_readings (
            id INTEGER PRIMARY KEY,
            timestamp TEXT,
            instantaneous_power REAL,
            min_power REAL,
            max_power REAL,
            avg_power REAL,
            sample_period TEXT,
            power_state TEXT,
            period TEXT CHECK(period IN ('now', '1_hour')) NOT NULL
        )
        """
        )

        conn.commit()
        conn.close()

    except sqlite3.Error as e:
        print(f"Error creating or connecting to the database: {e}")


def extract_power_data(parsed_data, node_id, period, timestamp):
    """Extract and convert power values ​​from dictionary"""

    def extract_value(key, default="0 Watts"):
        """Extracts the number from a string with format '<value> Watts'."""
        value = parsed_data.get(key, default)
        return (
            float(re.search(r"[\d.]+", value).group())
            if re.search(r"[\d.]+", value)
            else 0.0
        )

    return PowerReading(
        node_id=node_id,
        instantaneous_power=extract_value("Instantaneous power reading"),
        min_power=extract_value("Minimum during sampling period"),
        max_power=extract_value("Maximum during sampling period"),
        avg_power=extract_value("Average power reading over sample period"),
        sample_period=parsed_data.get("Sampling period", "0")
        .replace("Seconds.", "")
        .strip(),
        power_state=parsed_data.get("Power reading state is", ""),
        period=period,
        measurement_time=timestamp,
    )



import platform
import subprocess
from datetime import datetime, timezone
from src.modules.utilities import parse_and_log_ipmi_output, save_to_db


def run_ipmitool(sample_period=None):
    """Execute ipmitool and return the output as string."""

    if "microsoft" in platform.uname().release:  # Detecta si está en WSL
        return mock_ipmitool_output(sample_period)
    command = ["ipmitool", "dcmi", "power", "reading"]
    if sample_period:
        command.append(sample_period)

    try:
        result = subprocess.run(command, text=True, capture_output=True, check=True)
        return result.stdout
    except subprocess.CalledProcessError as e:
        print(f"Error executing ipmitool: {e}")
        print(f"Error: {e.stderr}")
        return None


def mock_ipmitool_output(sample_period):
    """Simula la salida de ipmitool con valores de prueba."""
    return """\
    Instantaneous power reading:                   330 Watts
    Minimum during sampling period:                  5 Watts
    Maximum during sampling period:                629 Watts
    Average power reading over sample period:      330 Watts
    IPMI timestamp:                           05/02/2025 11:25:41 PM UTC    Sampling period:                          00000001 Seconds.
    Power reading state is:                   activated
"""


def main():
    """Perform power measurements and save data to database."""
    timestamp = datetime.now(timezone.utc).isoformat(timespec="seconds")

    for sample_period_label in ["now", "1_hour"]:
        ipmi_output = run_ipmitool(
            None if sample_period_label == "now" else sample_period_label
        )
        if ipmi_output:
            parsed_data = parse_and_log_ipmi_output(
                ipmi_output, sample_period_label, timestamp
            )

            save_to_db(parsed_data, sample_period_label, timestamp)

    print("Measurement process completed!")



from setuptools import setup, find_packages

with open("README.md", "r") as f:
    description = f.read()

setup(
    name="power_corn",
    version="0.5",
    packages=find_packages(),
    install_requires=["supabase"],
    entry_points={
        "console_scripts": ["power_corn = src.main:main"],
    },
    long_description=description,
    long_description_content_type="text/markdown",
)
